/*
    Copyright (c) 2013 aizo ag, Zurich, Switzerland

    This file is part of libdsuid.

    libdsuid is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    libdsuid is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __LIBDSUID_H__
#define __LIBDSUID_H__

#include <stdbool.h>
#include "digitalSTROM/ds.h"

#ifdef __cplusplus
extern "C" {
#endif

/* return codes */
typedef enum {
    DSUID_RC_OK         = 0,  /* success                  */
    DSUID_RC_INV_DSUID  = 1,  /* dsuid can not be used    */
    DSUID_RC_INV_SERIAL = 2,  /* unexpected serial number */
    DSUID_RC_GEN_FAIL   = 3,  /* general system failure   */
    DSUID_RC_WRONG_PAR  = 4,  /* wrong function paramater */
    DSUID_RC_UUID_ERR   = 5,  /* error from ossp uuid lib */
} dsuid_rc_t;

/* namespaces */
typedef enum {
    DSUID_NS_ENOCEAN,
    DSUID_NS_GS1_128,
    DSUID_NS_IEEE_MAC,
    DSUID_NS_VDSM,
    DSUID_NS_VDC,
} dsuid_ns_t;


int dsuid_generate_v1(dsuid_t *dsuid);
int dsuid_generate_v1_mac(const char *mac, dsuid_t *dsuid);
int dsuid_generate_v3_from_namespace(dsuid_ns_t ns, const char *name, dsuid_t *dsuid);
int dsuid_generate_v4_random(dsuid_t *dsuid);

const char *dsuid_rc_to_text(int rc);
dsuid_t dsuid_from_dsid(const dsid_t *dsid);
int dsuid_to_dsid(const dsuid_t *dsuid, dsid_t *dsid);
int dsuid_get_next_dsuid(const dsuid_t *dsuid, dsuid_t *dsuid_next);
int dsuid_get_serial_number(const dsuid_t *dsuid, uint32_t *serialNumber);

/* see DSUID_STR_LEN */
int dsuid_from_string(const char *buf, dsuid_t *dsuid);
int dsuid_to_string(const dsuid_t *dsuid, char *buf);

bool dsuid_is_null(const dsuid_t *dsuid);
bool dsuid_equal(const dsuid_t *dsuid1, const dsuid_t *dsuid2);

#ifdef __cplusplus
}
#endif


#endif
